// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;
const St = imports.gi.St;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const PanelTools = Me.imports.panelTools;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

const KEY_SHOW_NOTIFICATIONS = 'show-notifications';

const PopupNotificationMenuItem = new Lang.Class({
    Name: 'PopupNotificationMenuItem',
    Extends: PanelTools.PopupMenuItem,

    _init: function(notification) {
        this._notification = notification;
        this.parent(this._notification.title, true);
        this._setIconName();
        this._setStatus();
        this.addButton('window-close-symbolic', Lang.bind(this, function() {
            this._notification.destroy();
        }));
        this._connectNotification = [
            this._notification.connect('destroy', Lang.bind(this, this.destroy)),
            this._notification.connect('icon-updated', Lang.bind(this, this._setIconName)),
            this._notification.connect('title-changed', Lang.bind(this, this._setStatus)),
            this._notification.connect('updated', Lang.bind(this, function() {
                this._setIconName();
                this._setStatus();
            }))
        ];
        this.connect('activate', Lang.bind(this, function() {
            Main.overview.hide();
            if (this._notification.activate) {
                this._notification.activate();
            } else if (this._notification._mount)  {
                let event = Clutter.get_current_event();
                let launchContext = global.create_app_launch_context(event.get_time(), -1);
                try { Gio.app_info_launch_default_for_uri(this._notification._mount.get_root().get_uri(), launchContext); } catch(e) {}
                this._notification.destroy();
            } else if (this._notification.source && this._notification.source._app) {
                this._notification.source._app.activate();
            }
        }));
        this.connect('destroy', Lang.bind(this, function() {
            for each(let connection in this._connectNotification) {
                this._notification.disconnect(connection);
            }
        }));
    },

    _getIconName: function() {
        let iconNames = null;
        if (!iconNames) try { iconNames = this._notification._mount.get_icon().get_names(); } catch(e) {}
        if (!iconNames) try { iconNames = this._notification.gicon.get_names(); } catch(e) {}
        if (!iconNames) try { iconNames = this._notification.source._app.get_app_info().get_icon().get_names(); } catch(e) {}
        for each(let iconName in iconNames) {
            if (Gtk.IconTheme.get_default().has_icon(iconName)) return iconName;
        }
        return iconNames.toString() || 'dialog-information';
    },

    _setIconName: function() {
        this.setIconName(this._getIconName());
    },

    _setStatus: function() {
        let status = this._notification._bannerBodyText || this._notification.bannerBodyText || '';
        this.setStatus(PanelTools.getStringCut(status, 50));
    }
});

const NotificationsButton = new Lang.Class({
    Name: 'NotificationsButton',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.0, _("Notifications"), { icon_name: 'dialog-warning' });
        this._sectionNotifications = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._sectionNotifications);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemClear = new PanelTools.PopupMenuItem(_("Clear Notifications"), true);
        this._itemClear.setIconName('edit-clear');
        this.menu.addMenuItem(this._itemClear);
        this._itemClear.connect('activate', Lang.bind(this, function() {
            if (this.menu.isOpen) this.menu.close(0);
            Main.overview.hide();
            this.actor.hide();
            this._sectionNotifications.removeAll();
        }));
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_NOTIFICATIONS, Lang.bind(this, this._onShowNotifications));
        this._connectMessageTray = [
            Main.messageTray.connect('source-added', Lang.bind(this, this._onShowNotifications)),
            //Main.messageTray.connect('source-removed', Lang.bind(this, this._onShowNotifications)),
            Main.messageTray.connect('queue-changed', Lang.bind(this, this._onShowNotifications))
        ];
        this._onShowNotifications();
        this.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (open) this.icon.icon_name = 'dialog-warning-symbolic';
        }));
        this.actor.connect('destroy', Lang.bind(this, function() {
            for each(let connection in this._connectMessageTray) {
                Main.messageTray.disconnect(connection);
            }
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _onShowNotifications: function() {
        let visible = this._settings.get_boolean(KEY_SHOW_NOTIFICATIONS);
        if (visible) {
            this._updateNotifications();
            visible = this._sectionNotifications.numMenuItems > 0;
        }
        this.updateVisibility(visible);
    },

    _updateNotifications: function() {
        let sources = Main.messageTray._summaryItems || Main.messageTray.getSources();
        for each(let source in sources) {
            if (source.source) source = source.source;
            if (source._mounts) continue;
            for each(let notification in source.notifications) {
                let addItem = true;
                let menuItems = this._sectionNotifications._getMenuItems();
                for each(let item in menuItems) {
                    let status = PanelTools.getStringCut(notification._bannerBodyText || notification.bannerBodyText || '', 50);
                    if (item.label.text == notification.title && item.status.text == status) addItem = false;
                }
                if (addItem) {
                    let item = new PopupNotificationMenuItem(notification);
                    item.actor.connect('destroy', Lang.bind(this, function() {
                        if (this.menu.isOpen) this.menu.close(0);
                        if (this._sectionNotifications.numMenuItems == 1) this.actor.hide();
                    }));
                    this._sectionNotifications.addMenuItem(item, 0);
                    this.icon.icon_name = 'dialog-warning';
                }
            }
        }
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfNotifications', new NotificationsButton(), PanelTools.getNumChildren('right') - 1, 'right');
}
