// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

const KEY_BOOKMARKS_SCOPE = 'bookmarks-scope';
const KEY_HOTCORNERS_SENSITIVITY = 'hotcorners-sensitivity';
const KEY_MENUS_HAVE_ICONS = 'menus-have-icons';
const KEY_MENUS_ICON_SIZE = 'menus-icon-size';
const KEY_SHOW_ACCESSIBILITY = 'show-accessibility';
const KEY_SHOW_AIRPLANEMODE = 'show-airplanemode';
const KEY_SHOW_ACTIVITIES = 'show-activities';
const KEY_SHOW_APPLICATIONS = 'show-applications';
const KEY_SHOW_BATTERY = 'show-battery';
const KEY_SHOW_BLUETOOTH = 'show-bluetooth';
const KEY_SHOW_BOOKMARKS = 'show-bookmarks';
const KEY_SHOW_BOOKMARKS_CUSTOM = 'show-bookmarks-custom';
const KEY_SHOW_BRIGHTNESS = 'show-brightness';
const KEY_SHOW_DASH = 'show-dash';
const KEY_SHOW_DATEMENU = 'show-datemenu';
const KEY_SHOW_DROPDOWN_ARROW = 'show-dropdown-arrow';
const KEY_SHOW_FAVORITES = 'show-favorites';
const KEY_SHOW_GNOME_GLOBE = 'show-gnome-globe';
const KEY_SHOW_GNOME_SEARCH_TOOL = 'show-gnome-search-tool';
const KEY_SHOW_HOTCORNER_LEFT = 'show-hotcorner-left';
const KEY_SHOW_HOTCORNER_RIGHT = 'show-hotcorner-right';
const KEY_SHOW_LOCATION = 'show-location';
const KEY_SHOW_KEYBOARD = 'show-keyboard';
const KEY_SHOW_MESSAGETRAY_APPTITLE = 'show-messagetray-apptitle';
const KEY_SHOW_NETWORK = 'show-network';
const KEY_SHOW_NETWORK_PLACES = 'show-network-places';
const KEY_SHOW_NOTIFICATIONS = 'show-notifications';
const KEY_SHOW_PLACES = 'show-places';
const KEY_SHOW_RECENT_DOCUMENTS = 'show-recent-documents';
const KEY_SHOW_REMOVABLE_DEVICES = 'show-removable-devices';
const KEY_SHOW_SHOWDESKTOP = 'show-showdesktop';
const KEY_SHOW_STATUSAREA = 'show-statusarea';
const KEY_SHOW_STATUSMENU = 'show-statusmenu';
const KEY_SHOW_TRASH = 'show-trash';
const KEY_SHOW_VOLUME = 'show-volume';

const SETTINGS_GLOBAL = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_ACTIVITIES,
        label:   _("Show Activities"),
        tooltip: _("If true, display Activities in GNOME Shell Panel.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_HOTCORNER_LEFT,
        label:   _("Activate left HotCorner"),
        tooltip: _("If true, activate HotCorner on left side in GNOME Shell Panel.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_HOTCORNER_RIGHT,
        label:   _("Activate right HotCorner"),
        tooltip: _("If true, activate HotCorner on right side in GNOME Shell Panel.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'slider',
        key:     KEY_HOTCORNERS_SENSITIVITY,
        label:   _("Response sensitivity for HotCorner"),
        tooltip: _("Setting the response sensitivity for HotCorner."),
        size:    150,
        min:     1,
        max:     10,
        step:    1,
        default: 1
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_DASH,
        label:   _("Show Dash"),
        tooltip: _("If true, display Dash otherwise not.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_MENUS_HAVE_ICONS,
        label:   _("Menus Have Icons"),
        tooltip: _("Whether menus may display on icon next to a menu entry in PanelManager.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'slider',
        key:     KEY_MENUS_ICON_SIZE,
        label:   _("Icon Size for menus"),
        tooltip: _("Setting the size on icon next to a menu entry in PanelManager."),
        size:    150,
        min:     16,
        max:     24,
        step:    1,
        default: 18
    }
];
if (Convenience.checkGSV(3, 4)) SETTINGS_GLOBAL.splice(5 , 0, {
    schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
    type:    'switcher',
    key:     KEY_SHOW_MESSAGETRAY_APPTITLE,
    label:   _("Show application title in MessageTray"),
    tooltip: _("If true, display application title in GNOME Shell MessageTray.")
});
if (!Convenience.checkGSV(3, 4) && !Convenience.checkGSV(3, 6) && !Convenience.checkGSV(3, 8)) SETTINGS_GLOBAL.splice(5 , 0, {
    schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
    type:    'switcher',
    key:     KEY_SHOW_DROPDOWN_ARROW,
    label:   _("Show DropDown-Arrow"),
    tooltip: _("If true, display buttons with DropDown-Arrow in GNOME Shell panel.")
});

const SETTINGS_APPLICATIONS = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_APPLICATIONS,
        label:   _("Show Applications"),
        tooltip: _("Select the display mode for Applications."),
        values:  { 'none': _("Not display"),
                   'all':  _("Icon and Text"),
                   'icon': _("Icon only"),
                   'text': _("Text only") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_FAVORITES,
        label:   _("Show Favorites"),
        tooltip: _("Select the display mode for Favorites."),
        values:  { 'none':    _("Not display"),
                   'all':     _("Icon and Text"),
                   'icon':    _("Icon only"),
                   'text':    _("Text only") }
    },
    {   schema:  'org.gnome.shell',
        type:    'tree',
        key:     'favorite-apps',
        label:   _("Favorites"),
        buttons: { 'add': _("Add Favorite"),
                   'del': _("Remove Favorite") },
        dialog:  'Favorites'
    }
];
if (GLib.file_test('/usr/bin/alacarte', GLib.FileTest.EXISTS)) SETTINGS_APPLICATIONS.splice(1, 0, {
    type:    'button',
    label:   _("Change which applications are shown"),
    tooltip: _("Change which applications are shown on the main menu"),
    value:   _("Main Menu"),
    app:     'alacarte'
});

const SETTINGS_PLACES = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_PLACES,
        label:   _("Show Places"),
        tooltip: _("Select the display mode for Places."),
        values:  { 'none': _("Not display"),
                   'all':  _("Icon and Text"),
                   'icon': _("Icon only"),
                   'text': _("Text only") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_BOOKMARKS_SCOPE,
        label:   _("Bookmarks scope"),
        tooltip: _("Determine the scope of Bookmarks in Places menu."),
        values:  { 'default': _("Default"),
                   'all':     _("All") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_BOOKMARKS,
        label:   _("Place of Bookmarks"),
        tooltip: _("Select the display mode for Bookmarks in Places menu."),
        values:  { 'none':     _("Not display"),
                   'submenu':  _("Display in sub menu"),
                   'mainmenu': _("Display in main menu") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_BOOKMARKS_CUSTOM,
        label:   _("Place of user defined Bookmarks"),
        tooltip: _("Select the display mode for user defined Bookmarks in Places menu."),
        values:  { 'none':     _("Not display"),
                   'submenu':  _("Display in sub menu"),
                   'mainmenu': _("Display in main menu") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_REMOVABLE_DEVICES,
        label:   _("Show Removable Devices"),
        tooltip: _("If true, display Removable Devices menu entry in Places menu.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_NETWORK_PLACES,
        label:   _("Show Network Places"),
        tooltip: _("If true, display the Network Places menu entry in Places menu.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_RECENT_DOCUMENTS,
        label:   _("Show Recent Documents"),
        tooltip: _("If true, display Recent Documents in menu entry Places menu.")
    }
];
if (GLib.file_test('/usr/bin/gnome-search-tool', GLib.FileTest.EXISTS)) SETTINGS_PLACES.splice(6, 0, {
    schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
    type:    'switcher',
    key:     KEY_SHOW_GNOME_SEARCH_TOOL,
    label:   _("Show Search for Files..."),
    tooltip: _("If true, display Search for Files... menu entry in Places menu.")
});
if (GLib.file_test('/usr/bin/nautilus-connect-server', GLib.FileTest.EXISTS)) SETTINGS_PLACES.splice(6, 0, {
    schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
    type:    'switcher',
    key:     KEY_SHOW_GNOME_GLOBE,
    label:   _("Show Connect to Server..."),
    tooltip: _("If true, display Connect to Server... menu entry in Places menu.")
});

const SETTINGS_DATEMENU = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_DATEMENU,
        label:   _("Show Time and Date"),
        tooltip: _("If true, display Time and Date in GNOME Shell Panel.")
    },
    {   schema:  Convenience.checkGSV(3, 4) ? 'org.gnome.shell.clock' : 'org.gnome.desktop.interface',
        type:    'switcher',
        key:     Convenience.checkGSV(3, 4) ? 'show-date' : 'clock-show-date',
        label:   _("Show date in clock"),
        tooltip: _("If true, display date in the clock, in addition to time.")
    },
    {   schema:  Convenience.checkGSV(3, 4) ? 'org.gnome.shell.clock' : 'org.gnome.desktop.interface',
        type:    'switcher',
        key:     Convenience.checkGSV(3, 4) ? 'show-seconds' : 'clock-show-seconds',
        label:   _("Show time with seconds"),
        tooltip: _("If true, display seconds in time.")
    },
    {   schema:  Convenience.checkGSV(3, 22) ? 'org.gnome.desktop.calendar' : 'org.gnome.shell.calendar',
        type:    'switcher',
        key:     'show-weekdate',
        label:   _("Show the week date in the calendar"),
        tooltip: _("If true, display the ISO week date in the calendar.")
    }
];

const SETTINGS_STATUSAREA = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_STATUSAREA,
        label:   _("StatusArea display mode"),
        tooltip: _("Determine the display mode for the StatusArea."),
        values:  { 'default':   _("Default"),
                   'advanced':  _("Advanced"),
                   'alternate': _("Alternate") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_ACCESSIBILITY,
        label:   _("Show Accessibility"),
        tooltip: _("Select the display mode for Accessibility."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_KEYBOARD,
        label:   _("Show Keyboard"),
        tooltip: _("Select the display mode for Keyboard."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_STATUSMENU,
        label:   _("Show StatusMenu"),
        tooltip: _("Select the display mode for StatusMenu."),
        values:  { 'default': _("Default"),
                   'icon':    _("Power icon") }
    }
];
if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
    SETTINGS_STATUSAREA.splice(3, 0, {
        schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_VOLUME,
        label:   _("Show Volume"),
        tooltip: _("Select the display mode for Volume."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    });
    SETTINGS_STATUSAREA.splice(4, 0, {
        schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_BLUETOOTH,
        label:   _("Show Bluetooth"),
        tooltip: _("Select the display mode for Bluetooth."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    });
    SETTINGS_STATUSAREA.splice(5, 0, {
        schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_NETWORK,
        label:   _("Show Network"),
        tooltip: _("Select the display mode for Network."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    });
    SETTINGS_STATUSAREA.splice(6, 0, {
        schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_BATTERY,
        label:   _("Show Battery"),
        tooltip: _("Select the display mode for Battery."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    });
} else {
const SETTINGS_STATUSMENU = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_VOLUME,
        label:   _("Show Volume"),
        tooltip: _("Select the display mode for Volume."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_BRIGHTNESS,
        label:   _("Show Brightness"),
        tooltip: _("Select the display mode for Brightness."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_NETWORK,
        label:   _("Show Network"),
        tooltip: _("Select the display mode for Network."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_BLUETOOTH,
        label:   _("Show Bluetooth"),
        tooltip: _("Select the display mode for Bluetooth."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_AIRPLANEMODE,
        label:   _("Show AirplaneMode"),
        tooltip: _("Select the display mode for AirplaneMode."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_BATTERY,
        label:   _("Show Battery"),
        tooltip: _("Select the display mode for Battery."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    }
];
if (!Convenience.checkGSV(3, 10)) {
    SETTINGS_STATUSMENU.splice(4, 0, {
        schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_LOCATION,
        label:   _("Show Location"),
        tooltip: _("Select the display mode for Location."),
        values:  { 'default': _("Default"),
                   'show':    _("Display"),
                   'none':    _("Not display") }
    });
}
}

const SETTINGS_OTHERS = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_SHOWDESKTOP,
        label:   _("Show ShowDesktop"),
        tooltip: _("If true, display ShowDesktop in GNOME Shell Panel.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_TRASH,
        label:   _("Show Trash"),
        tooltip: _("If true, display Trash in GNOME Shell Panel.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_NOTIFICATIONS,
        label:   _("Show Notifications"),
        tooltip: _("If true, display Notifications in GNOME Shell Panel.")
    }
];

const PanelManagerPrefsWidget = new GObject.Class({
    Name: 'PanelManager.Prefs.Widget',
    GTypeName: 'PanelManagerPrefsWidget',
    Extends: Gtk.Notebook,

    _init: function(params) {
        this.parent(params);
        this._createPrefsWidget();
    },

    _createPrefsWidget: function() {
        let box;
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (let i in SETTINGS_GLOBAL) {
            box.add(this._createWidget(SETTINGS_GLOBAL[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("General") }));
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (let i in SETTINGS_APPLICATIONS) {
            box.add(this._createWidget(SETTINGS_APPLICATIONS[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("Applications") }));
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (let i in SETTINGS_PLACES) {
            box.add(this._createWidget(SETTINGS_PLACES[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("Places") }));
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (let i in SETTINGS_DATEMENU) {
            box.add(this._createWidget(SETTINGS_DATEMENU[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("Time and Date") }));
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (let i in SETTINGS_STATUSAREA) {
            box.add(this._createWidget(SETTINGS_STATUSAREA[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("StatusArea") }));
        if (!Convenience.checkGSV(3, 4) && !Convenience.checkGSV(3, 6) && !Convenience.checkGSV(3, 8)) {
            box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
            for (let i in SETTINGS_STATUSMENU) {
                box.add(this._createWidget(SETTINGS_STATUSMENU[i]));
            }
            this.append_page(box, new Gtk.Label({ label: _("StatusMenu") }));
        }
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (let i in SETTINGS_OTHERS) {
            box.add(this._createWidget(SETTINGS_OTHERS[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("Others") }));
    },

    _createWidget: function(args) {
        let box;
        let settings = Convenience.getSettings(args['schema']);
        let widget;
        if (args['type'] == 'tree') {
            box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
            widget = new Gtk.Frame();
            box.add(widget);
            let scrolledWindow = new Gtk.ScrolledWindow({ expand: true });
            scrolledWindow.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
            widget.add(scrolledWindow);
            let listStore = new Gtk.ListStore();
            listStore.set_column_types([ GObject.TYPE_STRING, GObject.TYPE_STRING ]);
            let treeView = new Gtk.TreeView({ expand: true, model: listStore });
            scrolledWindow.add(treeView);
            treeView.get_selection().set_mode(Gtk.SelectionMode.SINGLE);
            let columnFavorites = new Gtk.TreeViewColumn({ expand: true, sort_column_id: 1, title: _(args['label']) });
            let textRenderer = new Gtk.CellRendererText();
            columnFavorites.pack_start(textRenderer, true);
            columnFavorites.add_attribute(textRenderer, 'text', 1);
            treeView.insert_column(columnFavorites, 0);
            if (args['buttons']) {
                let toolbar = new Gtk.Toolbar();
                toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR);
                box.add(toolbar);
                if (args['buttons']['add']) {
                    let buttonADD = new Gtk.ToolButton({ icon_name: 'list-add', label: _(args['buttons']['add']), is_important: true });
                    buttonADD.connect('clicked', Lang.bind(this, function() {
                        if (args['dialog'] == 'Favorites') this._searchFavorites(settings, args['key']);
                    }));
                    toolbar.add(buttonADD);
                }
                if (args['buttons']['del']) {
                    let buttonDEL = new Gtk.ToolButton({ icon_name: 'list-remove', label: _(args['buttons']['del']), is_important: true });
                    buttonDEL.connect('clicked', function() {
                        strArray = settings.get_strv(args['key']);
                        let [ any, model, iter ] = treeView.get_selection().get_selected();
                        if (any && strArray.length > 1) {
                            let item = listStore.get_value(iter, 0);
                            let index = strArray.map(function(i) {
                                return i;
                            }).indexOf(item);
                            if (index < 0) return;
                            strArray.splice(index, 1);
                            settings.set_strv(args['key'], strArray);
                        }
                    });
                    toolbar.add(buttonDEL);
                }
            }
            let strArray = [];
            let treeViewSelection = treeView.get_selection();
            let updateListStore = function() {
                strArray = settings.get_strv(args['key']);
                strArray.sort(function(a, b) {
                    try {
                        return Convenience.getSorted(Gio.DesktopAppInfo.new(a).get_name(), Gio.DesktopAppInfo.new(b).get_name());
                    } catch(e) {
                        return Convenience.getSorted(a, b);
                    }
                });
                listStore.clear();
                let item;
                for (let i = 0; i < strArray.length; i++) {
                    try {
                        item = Gio.DesktopAppInfo.new(strArray[i]).get_name();
                    } catch(e) {
                        item = strArray[i];
                    }
                    listStore.set(listStore.append(), [ 0, 1 ], [ strArray[i], item ]);
                }
                treeViewSelection.select_path(Gtk.TreePath.new_from_string(String(0)));
            }
            updateListStore();
            settings.connect('changed::'+args['key'], updateListStore);
        } else {
            box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
            box.pack_start(new Gtk.Label({ label: _(args['label']), xalign: 0 }), true, true, 0);
            switch (args['type']) {
                case 'button':
                    widget = new Gtk.Button({ label: _(args['value']) });
                    widget.height_request = 30;
                    widget.width_request = 200;
                    widget.connect('pressed', function() {
                        let [success, argv] = GLib.shell_parse_argv(args['app']);
                        GLib.spawn_async(null, argv, null, GLib.SpawnFlags.SEARCH_PATH, null, null);
                    });
                    break;
                case 'combo':
                    widget = new Gtk.ComboBoxText();
                    for (let i in args['values']) {
                        widget.append(i, _(args['values'][i]));
                    }
                    widget.set_active_id(settings.get_string(args['key']));
                    widget.width_request = 200;
                    widget.connect('changed', function() {
                        settings.set_string(args['key'], widget.get_active_id());
                    });
                    settings.connect('changed::'+args['key'], function() {
                        widget.set_active_id(settings.get_string(args['key']));
                    });
                    break;
                case 'slider':
                    let widgetTimeout = 0;
                    widget =  new Gtk.Scale({ orientation: Gtk.Orientation.HORIZONTAL, valuePos: Gtk.PositionType.LEFT });
                    widget.set_range(args['min'], args['max']);
                    widget.set_value(settings.get_int(args['key']));
                    widget.set_digits(0);
                    widget.set_increments(1, 1);
                    widget.set_size_request(args['size'], -1);
                    widget.connect('value-changed', function() {
                        if (widgetTimeout > 0) Mainloop.source_remove(widgetTimeout);
                        widgetTimeout = Mainloop.timeout_add(250, function() {
                            settings.set_int(args['key'], widget.get_value());
                        });
                    });
                    settings.connect('changed::'+args['key'], function() {
                        widget.set_value(settings.get_int(args['key']));
                    });
                    break;
                case 'switcher':
                    widget = new Gtk.Switch({ active: settings.get_boolean(args['key']) });
                    widget.connect('notify::active', function() {
                        settings.set_boolean(args['key'], widget.get_active());
                    });
                    settings.connect('changed::'+args['key'], function() {
                        widget.set_active(settings.get_boolean(args['key']));
                    });
                    break;
            }
            if (args['tooltip']) widget.set_tooltip_text(_(args['tooltip']));
            box.pack_end(widget, false, false, 0);
        }
        return box;
    },

    _searchFavorites: function(schema, key) {
        let dialog;
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10)) {
            dialog = new Gtk.Dialog({ title: _("Add Favorite"), modal: true, resizable: false });
        } else {
            dialog = new Gtk.Dialog({ title: _("Add Favorite"), modal: true, resizable: false, transient_for: this.get_toplevel(), use_header_bar: true });
        }
        dialog.add_button(_("Cancel"), Gtk.ResponseType.CANCEL);
        let buttonOK = dialog.add_button(_("OK"), Gtk.ResponseType.OK);
        buttonOK.sensitive = 0;
        dialog.set_default_response(Gtk.ResponseType.OK);
        let grid = new Gtk.Grid({ margin: 15 });
        dialog.get_content_area().add(grid);
        let appChooser = new Gtk.AppChooserWidget({ expand: true });
        grid.attach(appChooser, 0, 0, 1, 1);
        appChooser.set_default_text('');
        appChooser.set_show_all(true);
        appChooser.connect('application-selected', function() {
            buttonOK.sensitive = 0;
            let appInfo = appChooser.get_app_info();
            if (!appInfo) return;
            let strArray = schema.get_strv(key);
            let app = appInfo.get_id();
            for (let i = 0; i <= strArray.length; i++) {
                if (strArray[i] == app) return;
            }
            buttonOK.sensitive = 1;
        });
        dialog.connect('response', function(dialog, response_id) {
            if (response_id == Gtk.ResponseType.OK) {
                let appInfo = appChooser.get_app_info();
                if (!appInfo) return;
                let strArray = schema.get_strv(key);
                strArray.push(appInfo.get_id());
                schema.set_strv(key, strArray);
            }
            dialog.destroy();
        });
        dialog.show_all();
    }
});

function init() {
    Convenience.initTranslations();
}

function buildPrefsWidget() {
    let widget = new PanelManagerPrefsWidget({
        scrollable: true,
        margin: 15,
        width_request: 720,
        height_request: 480
    });
    widget.show_all();
    return widget;
}
