// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Meta = imports.gi.Meta;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

function _canHibernate(asyncCallback) {
    if (this._proxy) {
        this._proxy.call("CanHibernate", null, Gio.DBusCallFlags.NONE, -1, null, function(proxy, asyncResult) {
            try {
                let result = proxy.call_finish(asyncResult).deep_unpack();
                asyncCallback(result[0] != 'no');
            } catch(e) {
                asyncCallback(false);
            }
        });
    } else {
        Mainloop.idle_add(Lang.bind(this, function() {
            asyncCallback(false);
            return false;
        }));
    }
}

function _updateHibernate(actor) {
    if (Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12) || Convenience.checkGSV(3, 14) || Convenience.checkGSV(3, 16) || Convenience.checkGSV(3, 18) || Convenience.checkGSV(3, 20)) {
        _canHibernate.call(this._loginManager, Lang.bind(this, function(result) {
            let disabled = Main.sessionMode.isLocked || (Main.sessionMode.isGreeter && this._loginScreenSettings.get_boolean('disable-restart-buttons'));
            actor.visible = result && !disabled;
        }));
    } else {
        _canHibernate.call(this._loginManager, Lang.bind(this, function(canHibernate, needsAuth) {
            let disabled = (Main.sessionMode.isLocked && needsAuth) || (Main.sessionMode.isGreeter && this._loginScreenSettings.get_boolean('disable-restart-buttons'));
            actor.visible = canHibernate && !disabled;
        }));
    }
}

function _getAggregateMenu() {
    try { return Main.panel.statusArea.aggregateMenu; } catch(e) { return null; }
}

function _getSystem() {
    try { return Main.panel.statusArea.aggregateMenu._system; } catch(e) { return null; }
}

const GNOME_AGGREGATEMENU = _getAggregateMenu();
const GNOME_SYSTEM = _getSystem();

const KEY_SHOW_STATUSAREA = 'show-statusarea';

const STATUSAREA_DEFAULT = 0;

let _prototypeUpdateMultiUser;

function _overrideFunction(enabled) {

    if (enabled) {
        _prototypeUpdateMultiUser = imports.ui.status.system.Indicator.prototype._updateMultiUser;
        GNOME_SYSTEM._updateMultiUser = function () {
            _prototypeUpdateMultiUser.call(this);
            let pmShowStatusArea = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_STATUSAREA);
            if (!this._switchUserSubMenu.actor.visible && pmShowStatusArea != STATUSAREA_DEFAULT) this._switchUserSubMenu.actor.visible = this._lockScreenAction.visible;
        };
    } else {
        GNOME_SYSTEM._updateMultiUser = _prototypeUpdateMultiUser;
    }
}

const SystemIndicator = new Lang.Class({
    Name: 'SystemIndicator',

    _init: function() {
        this._itemLockScreen = new PopupMenu.PopupMenuItem(_("Lock Screen"));
        this._itemLockScreen.connect('activate', Lang.bind(GNOME_SYSTEM, GNOME_SYSTEM._onLockScreenClicked));
        GNOME_SYSTEM._switchUserSubMenu.menu.addMenuItem(this._itemLockScreen, 1);
        this._actionPanelManagerSettings = this._addAction('preferences-other-symbolic', _("Settings for PanelManager"), function() {
            this.menu.itemActivated();
            Main.overview.hide();
            Main.Util.spawnCommandLine('gnome-shell-extension-prefs '+Me.metadata['uuid']);
        });
        this._actionGNOMEShellRestart = this._addAction('view-refresh-symbolic', _("GNOME Shell restart"), function() {
            Meta.restart ? Meta.restart(_("Restarting…")) : global.reexec_self();
        });
        this._actionTerminal = this._addAction('utilities-terminal-symbolic', _("Terminal"), function() {
            this.menu.itemActivated();
            Main.overview.hide();
            Main.Util.spawnCommandLine('gnome-terminal');
        });
        this._actionSuspend = this._addAction('media-playback-pause-symbolic', _("Suspend"), GNOME_SYSTEM._onSuspendClicked);
        this._actionHibernate = this._addAction('media-playback-stop-symbolic', _("Hibernate"), function() {
            this.menu.itemActivated();
            if (this._loginManager._proxy) {
                this._loginManager._proxy.call("Hibernate", GLib.Variant.new('(b)', [true]), Gio.DBusCallFlags.NONE, -1, null, null);
            } else {
                this._loginManager.emit('prepare-for-sleep', true);
                this._loginManager.emit('prepare-for-sleep', false);
            }
        });
        this._actionPowerOff = this._addAction('system-shutdown-symbolic', _("Power Off"), GNOME_SYSTEM._onPowerOffClicked);
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._onShowStatusArea));
        this._connectMenu = GNOME_AGGREGATEMENU.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (!open) return;
            this._itemLockScreen.actor.visible = GNOME_SYSTEM._lockScreenAction.visible;
            this._actionTerminal.visible = GLib.find_program_in_path('gnome-terminal');
            GNOME_SYSTEM._updateHaveSuspend();
            _updateHibernate.call(GNOME_SYSTEM, this._actionHibernate);
            GNOME_SYSTEM._updateHaveShutdown();
        }));
        _overrideFunction(true);
        this._onShowStatusArea();
    },

    _addAction: function(iconName, text, callback) {
        let item = GNOME_SYSTEM._createActionButton(iconName, _(text));
        item.connect('clicked', Lang.bind(GNOME_SYSTEM, callback));
        return item;
    },

    _onShowStatusArea: function() {
        this._updateVisibility(this._settings.get_enum(KEY_SHOW_STATUSAREA));
    },

    _updateVisibility: function(showStatusArea) {
        let children = GNOME_SYSTEM._actionsItem.actor.get_children();
        for (let i = 1; i < children.length; i++) {
            GNOME_SYSTEM._actionsItem.actor.remove_actor(children[i]);
        }
        GNOME_SYSTEM._actionsItem.actor.add(this._actionPanelManagerSettings, { expand: true, x_fill: false });
        if (showStatusArea == STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(GNOME_SYSTEM._settingsAction, { expand: true, x_fill: false });
        GNOME_SYSTEM._actionsItem.actor.add(GNOME_SYSTEM._orientationLockAction, { expand: true, x_fill: false });
        if (showStatusArea == STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(GNOME_SYSTEM._lockScreenAction, { expand: true, x_fill: false });
        if (showStatusArea == STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(GNOME_SYSTEM._altSwitcher.actor, { expand: true, x_fill: false });
        if (showStatusArea != STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(this._actionGNOMEShellRestart, { expand: true, x_fill: false });
        if (showStatusArea != STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(this._actionTerminal, { expand: true, x_fill: false });
        if (showStatusArea != STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(this._actionSuspend, { expand: true, x_fill: false });
        if (showStatusArea != STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(this._actionHibernate, { expand: true, x_fill: false });
        if (showStatusArea != STATUSAREA_DEFAULT) GNOME_SYSTEM._actionsItem.actor.add(this._actionPowerOff, { expand: true, x_fill: false });
        GNOME_SYSTEM._updateOrientationLock();
        GNOME_SYSTEM._sessionUpdated();
    },

    destroy: function() {
        GNOME_AGGREGATEMENU.menu.disconnect(this._connectMenu);
        this._settings.disconnect(this._connectSettings);
        _overrideFunction(false);
        this._updateVisibility(STATUSAREA_DEFAULT);
        this._actionPowerOff.destroy();
        this._actionHibernate.destroy();
        this._actionSuspend.destroy();
        this._actionTerminal.destroy();
        this._actionGNOMEShellRestart.destroy();
        this._actionPanelManagerSettings.destroy();
        this._itemLockScreen.destroy();
    }
});

function getExtension() {
    return GNOME_SYSTEM ? new SystemIndicator() : null;
}
