// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;
const Soup = imports.gi.Soup;
const St = imports.gi.St;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const PanelTools = Me.imports.panelTools;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

// Soup session (see https://bugzilla.gnome.org/show_bug.cgi?id=661323#c64) (Simon Legner)
const _httpSession = new Soup.SessionAsync();
Soup.Session.prototype.add_feature.call(_httpSession, new Soup.ProxyResolverDefault());

const KEY_LOCATION = 'location';
const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_WEATHER_FORECAST = 'show-weather-forecast';
const KEY_WEATHER_ICONS = 'weather-icons';
const KEY_WEATHER_LOCATIONS = 'weather-locations';
const KEY_WEATHER_UNITS = 'weather-units';
const KEY_WIND_DIRECTION = 'wind-direction';

//https://developer.yahoo.com/weather/documentation.html#codes
const WEATHER_CODE = [
    {   text: _("Tornado"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Tropical Storm "),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Hurricane"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Severe Thunderstorms"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Thunderstorms"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Mixed Rain and Snow"),
        icon: [ 'weather-snow-rain', 'weather-snow' ]
    },
    {   text: _("Mixed Rain and Sleet"),
        icon: [ 'weather-snow-rain', 'weather-snow' ]
    },
    {   text: _("Mixed Snow and Sleet"),
        icon: [ 'weather-snow-rain', 'weather-snow' ]
    },
    {   text: _("Freezing Drizzle"),
        icon: [ 'weather-freezing-rain', 'weather-showers' ]
    },
    {   text: _("Drizzle"),
        icon: [ 'weather-showers' ]
    },
    {   text: _("Freezing Rain"),
        icon: [ 'weather-freezing-rain', 'weather-showers' ]
    },
    {   text: _("Showers"),
        icon: [ 'weather-showers' ]
    },
    {   text: _("Showers"),
        icon: [ 'weather-showers' ]
    },
    {   text: _("Snow Flurries"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Light Snow Showers"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Blowing Snow"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Snow"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Hail"),
        icon: [ 'weather-showers' ]
    },
    {   text: _("Sleet"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Dust"),
        icon: [ 'weather-fog' ]
    },
    {   text: _("Foggy"),
        icon: [ 'weather-fog' ]
    },
    {   text: _("Haze"),
        icon: [ 'weather-fog' ]
    },
    {   text: _("Smoky"),
        icon: [ 'weather-fog' ]
    },
    {   text: _("Blustery"),
        icon: [ 'weather-few-clouds' ]
    },
    {   text: _("Windy"),
        icon: [ 'weather-few-clouds' ]
    },
    {   text: _("Cold"),
        icon: [ 'weather-few-clouds' ]
    },
    {   text: _("Cloudy"),
        icon: [ 'weather-overcast' ]
    },
    {   text: _("Mostly Cloudy"),
        icon: [ 'weather-few-clouds-night', 'weather-clouds-night' ]
    },
    {   text: _("Mostly Cloudy"),
        icon: [ 'weather-few-clouds', 'weather-clouds' ]
    },
    {   text: _("Partly Cloudy"),
        icon: [ 'weather-few-clouds-night', 'weather-clouds-night' ]
    },
    {   text: _("Partly Cloudy"),
        icon: [ 'weather-few-clouds' ]
    },
    {   text: _("Clear"),
        icon: [ 'weather-clear-night' ]
    },
    {   text: _("Sunny"),
        icon: [ 'weather-clear' ]
    },
    {   text: _("Fair"),
        icon: [ 'weather-clear-night' ]
    },
    {   text: _("Fair"),
        icon: [ 'weather-clear' ]
    },
    {   text: _("Mixed Rain and Hail"),
        icon: [ 'weather-showers' ]
    },
    {   text: _("Hot"),
        icon: [ 'weather-clear' ]
    },
    {   text: _("Isolated Thunderstorms"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Scattered Thunderstorms"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Scattered Showers"),
        icon: [ 'weather-showers-scattered', 'weather-showers' ]
    },
    {   text: _("Scattered Showers"),
        icon: [ 'weather-showers-scattered', 'weather-showers' ]
    },
    {   text: _("Heavy Snow"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Scattered Snow Showers"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Heavy Snow"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Partly Cloudy"),
        icon: [ 'weather-few-clouds' ]
    },
    {   text: _("Thundershowers"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Snow Showers"),
        icon: [ 'weather-snow' ]
    },
    {   text: _("Isolated Thundershowers"),
        icon: [ 'weather-storm' ]
    },
    {   text: _("Not Available"),
        icon: [ 'weather-severe-alert' ]
    }
];

const WEATHER_COMPASS_ARROWS = [
    '\u2193',
    '\u2199',
    '\u2190',
    '\u2196',
    '\u2191',
    '\u2197',
    '\u2192',
    '\u2198',
    '\u2193'
];

const WEATHER_COMPASS_LETTERS = [
    _('N'),
    _('NNE'),
    _('NE'),
    _('ENE'),
    _('E'),
    _('ESE'),
    _('SE'),
    _('SSE'),
    _('S'),
    _('SSW'),
    _('SW'),
    _('WSW'),
    _('W'),
    _('WNW'),
    _('NW'),
    _('NNW'),
    _('N')
];

const WEATHER_DAY = [
    _("Sunday"),
    _("Monday"),
    _("Tuesday"),
    _("Wednesday"),
    _("Thursday"),
    _("Friday"),
    _("Saturday")
];

const WEATHER_PRESSURE = [
    '',
    '\u2934',
    '\u2935'
];

const WeatherButton = new Lang.Class({
    Name: 'WeatherButton',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.25, _("Weather"), { icon_name: 'view-refresh-symbolic', text: '...' });
        this.event = null;
        this._interface = Convenience.getSettings('org.gnome.desktop.interface');
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._weather = null;
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_LOCATION, Lang.bind(this, this._onLocation)),
            this._settings.connect('changed::'+KEY_REFRESH_INTERVAL, Lang.bind(this, this._onRefreshInterval)),
            this._settings.connect('changed::'+KEY_SHOW_WEATHER_FORECAST, Lang.bind(this, this._onShowWeatherForecast)),
            this._settings.connect('changed::'+KEY_WEATHER_ICONS, Lang.bind(this, this._onWeatherIcons)),
            this._settings.connect('changed::'+KEY_WEATHER_LOCATIONS, Lang.bind(this, this._onWeatherLocations)),
            this._settings.connect('changed::'+KEY_WEATHER_UNITS, Lang.bind(this, this._onWeatherUnits)),
            this._settings.connect('changed::'+KEY_WIND_DIRECTION, Lang.bind(this, this._onWindDirection))
        ];
        this._createSubMenu();
        this._onWeatherLocations();
        this._updateSubMenu();
        this._updateWeather();
        this._connectInterface = this._interface.connect('changed::clock-format', Lang.bind(this, this._onClockFormat));
        this.actor.connect('destroy', Lang.bind(this, function() {
            if (this.event) Mainloop.source_remove(this.event);
            for each(let connection in this._connectSettings) {
                this._settings.disconnect(connection);
            }
            this._interface.disconnect(this._connectInterface);
        }));
    },

    _createSubMenu: function() {
        this._iconWeather = new St.Bin();
        this._labelBuild = new St.Label({ style_class: 'weathermenu-current-times-caption' });
        this._labelChill = new St.Label();
        this._labelHumidity = new St.Label();
        this._labelPressure = new St.Label();
        this._labelSummary = new St.Label({ style_class: 'weathermenu-current-summary' });
        this._labelSunrise = new St.Label({ style_class: 'weathermenu-current-times-caption' });
        this._labelSunset = new St.Label({ style_class: 'weathermenu-current-times-caption' });
        this._labelVisibility = new St.Label();
        this._labelWind = new St.Label();
        let iconBuild = new St.Icon({ icon_name: 'view-refresh', style_class: 'weathermenu-current-times-icon' });
        let iconSunrise = new St.Icon({ icon_name: 'weather-clear', style_class: 'weathermenu-current-times-icon' });
        let iconSunset = new St.Icon({ icon_name: 'weather-clear-night', style_class: 'weathermenu-current-times-icon' });
        if (St.IconType) {
            iconBuild.icon_type = St.IconType.FULLCOLOR;
            iconSunrise.icon_type = St.IconType.FULLCOLOR;
            iconSunset.icon_type = St.IconType.FULLCOLOR;
        }
        let box = new St.BoxLayout({ vertical: true, style_class: 'weathermenu-current' });
        let boxTimes = new St.BoxLayout();
        boxTimes.add(iconSunrise);
        boxTimes.add(this._labelSunrise);
        boxTimes.add(iconSunset);
        boxTimes.add(this._labelSunset);
        boxTimes.add(iconBuild);
        boxTimes.add(this._labelBuild);
        box.add(boxTimes);
        box.add(this._labelSummary);
        let boxCurrent = new St.BoxLayout();
        let boxIcon = new St.BoxLayout({ style_class: 'weathermenu-current-icon' });
        boxIcon.add(this._iconWeather);
        boxCurrent.add(boxIcon);
        let boxData = new St.BoxLayout({ vertical: true });
        let boxAverage = new St.BoxLayout({ style_class: 'weathermenu-current-average' });
        let boxAverageCaption = new St.BoxLayout({ vertical: true, style_class: 'weathermenu-current-average-caption' });
        boxAverageCaption.add(new St.Label({ text: _("Feels like:") }));
        boxAverageCaption.add(new St.Label({ text: _("Humidity:") }));
        boxAverageCaption.add(new St.Label({ text: _("Visibility:") }));
        boxAverageCaption.add(new St.Label({ text: _("Wind:") }));
        boxAverageCaption.add(new St.Label({ text: _("Pressure:") }));
        boxAverage.add(boxAverageCaption, { expand: true });
        let boxAverageValue = new St.BoxLayout({ vertical: true, style_class: 'weathermenu-current-average-value' });
        boxAverageValue.add(this._labelChill);
        boxAverageValue.add(this._labelHumidity);
        boxAverageValue.add(this._labelVisibility);
        boxAverageValue.add(this._labelWind);
        boxAverageValue.add(this._labelPressure);
        boxAverage.add(boxAverageValue);
        boxData.add(boxAverage, { expand: true });
        boxCurrent.add(boxData);
        box.add(boxCurrent);
        this._itemLocation = new PanelTools.PopupSubMenuMenuItem('...', false, { span: -1 });
        this._itemLocation.label.style_class = 'weathermenu-current-location';
        this.menu.addMenuItem(this._itemLocation);
        this._itemForecast = new PanelTools.PopupSubMenuMenuItem(_("Forecast"), false, { span: -1 });
        this.menu.addMenuItem(this._itemForecast);
        this._itemCurrent = new PopupMenu.PopupBaseMenuItem({ reactive: false });
        this._itemCurrent.addActor ? this._itemCurrent.addActor(box) : this._itemCurrent.actor.add(box);
        this.menu.addMenuItem(this._itemCurrent);
        this._itemActions = new PanelTools.PopupActionMenuItem({ span: -1 });
        this._itemActions.actor.style = 'padding-left: 0; padding-right: 0;';
        this._iconLogo = new St.Bin();
        this._actionLogo = this._itemActions.addActionButton(this._iconLogo, _("go to YAHOO"));
        this._actionLogo.style_class = 'weathermenu-yahoo-logo';
        this._actionLogo.connect('clicked', Lang.bind(this, this._showURI));
        this._actionSettings = this._itemActions.addActionButton('preferences-system-symbolic', _("Settings for Weather"));
        this._actionSettings.connect('clicked', Lang.bind(this, function() {
            if (this.menu.isOpen) this.menu.close(0);
            Main.overview.hide();
            Main.Util.spawnCommandLine('gnome-shell-extension-prefs '+Me.metadata['uuid']);
        }));
        this._actionRefresh = this._itemActions.addActionButton('view-refresh-symbolic', _("Refresh weather data"));
        this._actionRefresh.connect('clicked', Lang.bind(this, this._updateWeather));
        this.menu.addMenuItem(this._itemActions);
        this._itemActions.actor.reparent(boxData);
    },

    _createWeatherForecast: function() {
        this._itemForecast.menu.removeAll();
        this._weatherForecast = [];
        if (!this._weather) return;
        for (let i = 0; i < this._weather.item.forecast.length; i++) {
            let actualDate = new Date();
            let forecastDate = new Date(this._weather.item.forecast[i].date);
            let dayLeft = Math.floor((actualDate.getTime() - forecastDate.getTime()) / 1000 / 60 / 60 / 24);
            let date_string = _("Today");
            if (dayLeft == -1) {
                date_string = _("Tomorrow");
            } else if (dayLeft < -1) {
                date_string = _("In %s days").format(-1 * dayLeft);
            } else if (dayLeft == 1) {
                date_string = _("Yesterday");
            } else if (dayLeft > 1) {
                date_string = _("%s days ago").format(dayLeft);
            }
            this._weatherForecast[i] = {}
            this._weatherForecast[i].Day = new St.Label({ style_class: 'weathermenu-forecast-day' });
            this._weatherForecast[i].Day.set_text(date_string+' ('+this._getDate(this._weather.item.forecast[i].date)+')');
            this._weatherForecast[i].Icon = new St.Icon({ style_class: 'weathermenu-forecast-icon' });
            if (St.IconType) this._weatherForecast[i].Icon.icon_type = St.IconType.FULLCOLOR;
            this._weatherForecast[i].Summary = new St.Label({ style_class: 'weathermenu-forecast-summary' });
            this._weatherForecast[i].Summary.set_text(_(WEATHER_CODE[this._weather.item.forecast[i].code]['text']));
            this._weatherForecast[i].Temperature = new St.Label({ style_class: 'weathermenu-forecast-temperature' });
            let box = new St.BoxLayout({ style_class: 'weathermenu-forecast' });
            box.add(this._weatherForecast[i].Icon);
            let boxData = new St.BoxLayout({ vertical: true });
            boxData.add(this._weatherForecast[i].Day);
            let boxSummary = new St.BoxLayout();
            boxSummary.add(this._weatherForecast[i].Temperature);
            boxSummary.add(this._weatherForecast[i].Summary);
            boxData.add(boxSummary);
            box.add(boxData);
            let item = new PopupMenu.PopupBaseMenuItem({ reactive: false });
            item.addActor ? item.addActor(box) : item.actor.add(box);
            this._itemForecast.menu.addMenuItem(item);
        }
    },

    _getDate: function(date) {
        date = new Date(date+' 00:00');
        return _(WEATHER_DAY[date.getDay()])+' '+date.toLocaleDateString();
    },

    _getIconLogo: function() {
        if (!this._weather) {
            if (St.TextureCache.get_default().load_uri_async) return St.TextureCache.get_default().load_uri_async('https://poweredby.yahoo.com/purple.png', 134, 29, true);
            if (St.TextureCache.get_default().load_file_async) return St.TextureCache.get_default().load_file_async(Gio.file_new_for_uri('https://poweredby.yahoo.com/purple.png'), 134, 29, St.ThemeContext.get_for_stage(global.stage).scale_factor);
        } else {
            if (St.TextureCache.get_default().load_uri_async) return St.TextureCache.get_default().load_uri_async(this._weather.image.url, this._weather.image.width, this._weather.image.height, true);
            if (St.TextureCache.get_default().load_file_async) return St.TextureCache.get_default().load_file_async(Gio.file_new_for_uri(this._weather.image.url), this._weather.image.width, this._weather.image.height, St.ThemeContext.get_for_stage(global.stage).scale_factor);
        }
        return null;
    },

    _getIconName: function(code, symbolic) {
        if (code >= WEATHER_CODE.length) code = WEATHER_CODE.length - 1;
        let iconName = WEATHER_CODE[code]['icon'];
        let iconType = symbolic ? '-symbolic' : '';
        for (let i = 0; i < iconName.length; i++) {
            if (Gtk.IconTheme.get_default().has_icon(iconName[i]+iconType)) return iconName[i]+iconType;
        }
        return 'weather-severe-alert'+iconType;
    },

    _getIconWeather: function() {
        let icon;
        if (!this._weather) {
            icon = new St.Icon({ icon_name: 'view-refresh', style_class: 'weathermenu-current-icon-default' });
        } else {
            if (this._settings.get_string(KEY_WEATHER_ICONS) == 'default') {
                let currentDate = new Date();
                let sunriseDate = new Date(currentDate.toDateString()+' '+this._weather.astronomy.sunrise);
                let sunsetDate = new Date(currentDate.toDateString()+' '+this._weather.astronomy.sunset);
                let code = this._weather.item.condition.code;
                if (code > 43 && code < WEATHER_CODE.length - 1) {
                    code++;
                } else if (code == WEATHER_CODE.length - 1) {
                    code = 44;
                }
                let iconURI = 'http://l.yimg.com/a/i/us/nws/weather/gr/'+code+((currentDate >= sunriseDate && currentDate < sunsetDate) ? 'd' : 'n')+'.png';
                try {
                    if (St.TextureCache.get_default().load_uri_async) return St.TextureCache.get_default().load_uri_async(iconURI, 175, 126, true);
                    if (St.TextureCache.get_default().load_file_async) return St.TextureCache.get_default().load_file_async(Gio.file_new_for_uri(iconURI), 250, 180, St.ThemeContext.get_for_stage(global.stage).scale_factor);
                } catch (e) {}
            }
            icon = new St.Icon({ icon_name: this._getIconName(this._weather.item.condition.code, false), style_class: 'weathermenu-current-icon-default' });
        }
        if (St.IconType) icon.icon_type = St.IconType.FULLCOLOR;
        return icon;
    },

    _getLocation: function(param) {
        let locations = this._settings.get_strv(KEY_WEATHER_LOCATIONS);
        let i = this._settings.get_int(KEY_LOCATION);
        if (i >= locations.length) return (param = 0 ? '' : _("Invalid city"));
        return locations[i].split('>')[param];
    },

    _getPressure: function() {
        let pressure = parseFloat(this._weather.atmosphere.pressure);
        let unit = this._weather.units.pressure;
        if (this._settings.get_string(KEY_WEATHER_UNITS) == 'metric') {
            unit = 'hPa';
        } else {
            pressure = pressure * 0.02952998751;
        }
        return parseFloat(pressure.toFixed(2)).toLocaleString()+' '+unit+WEATHER_PRESSURE[parseInt(this._weather.atmosphere.rising)];
    },

    _getTemperature: function(temperature) {
        let temp = parseInt(temperature)
        let unit = this._weather.units.temperature;
        if (this._settings.get_string(KEY_WEATHER_UNITS) == 'metric') {
            temp = (temp - 32) / 1.8;
            unit = 'C';
        }
        return parseInt(temp).toString()+'\u00A0\u00B0'+unit;
    },

    _getTime: function(time) {
        if (this._interface.get_string('clock-format') == '24h' || this._settings.get_string(KEY_WEATHER_UNITS) == 'metric') {
            let currentDate = new Date();
            time = new Date(currentDate.toDateString()+' '+time);
            return time.getHours()+':'+(time.getMinutes() < 10 ? '0' : '')+time.getMinutes()+' Uhr';
        }
        return time.toLowerCase();
    },

    _getTimeBuild: function(time) {
        let timeBuild = time.split(' ');
        return this._getTime(timeBuild[4]+' '+timeBuild[5]);
    },

    _getVisibility: function() {
        let visibility = parseFloat(this._weather.atmosphere.visibility);
        let unit = this._weather.units.distance;
        if (visibility > 0) {
            if (this._settings.get_string(KEY_WEATHER_UNITS) == 'metric') {
                unit = 'km';
            } else {
                visibility = visibility / 1.60934;
            }
            return parseFloat(visibility.toFixed(2)).toLocaleString()+' '+unit;
        }
        return '?';
    },

    _getWind: function() {
        if (!this._weather.wind.speed) return '\u2013';
        let direction = '';
        let speed = parseInt(this._weather.wind.speed);
        let unit = this._weather.units.speed;
        if (!this._weather.wind.direction || speed == 0) {
        } else {
            if (this._settings.get_string(KEY_WIND_DIRECTION) == 'letters') {
                direction = ' '+WEATHER_COMPASS_LETTERS[Math.round(parseInt(this._weather.wind.direction) / 22.5)];
            } else {
                direction = ' '+WEATHER_COMPASS_ARROWS[Math.round(parseInt(this._weather.wind.direction) / 45)];
            }
        }
        if (this._settings.get_string(KEY_WEATHER_UNITS) == 'metric') {
            unit = 'km/h';
        } else {
            speed = speed / 1.60934;
        }
        return parseInt(speed).toString()+' '+unit+direction;
    },

    _loadWeather: function(query, callback) {
        let message = Soup.Message.new('GET', encodeURI('https://query.yahooapis.com/v1/public/yql?q='+query.toLowerCase()+'&format=json'));
        _httpSession.queue_message(message, Lang.bind(this, function(_httpSession, message) {
            try {
                callback.call(this, !message.response_body.data ? null : JSON.parse(message.response_body.data));
            } catch (e) {
                callback.call(this, null);
            }
        }));
    },

    _onClockFormat: function() {
        this._labelBuild.set_text(!this._weather ? '...' : this._getTimeBuild(this._weather.lastBuildDate)+' ('+this._getTimeBuild(this._weather.item.pubDate)+')');
        this._labelSunrise.set_text(!this._weather ? '...' : this._getTime(this._weather.astronomy.sunrise));
        this._labelSunset.set_text(!this._weather ? '...' : this._getTime(this._weather.astronomy.sunset));
    },

    _onLocation: function() {
        this._updateLocation();
        this._weather = null;
        this._updateSubMenu();
        this._updateWeather();
    },

    _onRefreshInterval: function() {
        this._updateWeather();
    },

    _onShowWeatherForecast: function() {
        this._itemForecast.actor.visible = this._itemForecast.menu.numMenuItems > 0 && this._settings.get_boolean(KEY_SHOW_WEATHER_FORECAST);
    },

    _onWeatherIcons: function() {
        this._iconWeather.child = this._getIconWeather(this._weather);
        for (let i = 0; i < this._weatherForecast.length; i++) {
            this._weatherForecast[i].Icon.icon_name = this._getIconName(this._weather.item.forecast[i].code, false);
        }
    },

    _onWeatherLocations: function() {
        this._itemLocation.menu.removeAll();
        let locations = this._settings.get_strv(KEY_WEATHER_LOCATIONS);
        for (let i = 0; i < locations.length; i++) {
            let item = new PanelTools.PopupMenuItem(locations[i].split('>')[1]);
            this._itemLocation.menu.addMenuItem(item);
            item.location = i;
            item.connect('activate', Lang.bind(this, function() {
                this._settings.set_int(KEY_LOCATION, item.location);
                item.setSelected(true);
                this._itemLocation.label.set_text(this._settings.get_strv(KEY_WEATHER_LOCATIONS)[item.location].split('>')[1]);
                if (this._itemLocation.menu.isOpen) this._itemLocation.menu.close(0);
                if (!this.menu.isOpen) this.menu.open(-1);
            }));
        }
        this._updateLocation();
    },

    _onWeatherUnits: function() {
        this.label.set_text(!this._weather ? '...' : this._getTemperature(this._weather.item.condition.temp));
        this._onClockFormat();
        this._labelSummary.set_text(!this._weather ? _("Loading weather data...") : _(WEATHER_CODE[this._weather.item.condition.code]['text'])+', '+this._getTemperature(this._weather.item.condition.temp));
        this._labelChill.set_text(!this._weather ? '...' : this._getTemperature(this._weather.wind.chill));
        this._labelHumidity.set_text(!this._weather ? '...' : this._weather.atmosphere.humidity+' %');
        this._labelVisibility.set_text(!this._weather ? '...' : this._getVisibility());
        this._onWindDirection();
        this._labelPressure.set_text(!this._weather ? '...' : this._getPressure());
        this._onWeatherIcons();
        for (let i = 0; i < this._weatherForecast.length; i++) {
            this._weatherForecast[i].Temperature.set_text('\u2191 '+this._getTemperature(this._weather.item.forecast[i].high)+'   \u2193 '+this._getTemperature(this._weather.item.forecast[i].low));
        }
    },

    _onWindDirection: function() {
        this._labelWind.set_text(!this._weather ? '...' : this._getWind());
    },

    _setRefreshInterval: function(interval) {
        if (this.event) Mainloop.source_remove(this.event);
        this.event = Mainloop.timeout_add_seconds(interval, Lang.bind(this, this._updateWeather));
    },

    _showURI: function () {
        let url = !this._weather ? 'https://www.yahoo.com/?ilc=401' : this._weather.item.link;
        if (this.menu.isOpen) this.menu.close(0);
        Main.overview.hide();
        try {
            Gtk.show_uri(null, url, global.get_current_time());
        } catch (err) {
            Main.notifyError(_("Can not open '%s'").format(url), err.message);
        }
    },

    _updateLocation: function() {
        let menuItems = this._itemLocation.menu._getMenuItems();
        if (this._itemLocation.selected)this._itemLocation.selected.setSelected(false);
        for each(let item in menuItems) {
            if (item.location == this._settings.get_int(KEY_LOCATION)) {
                this._itemLocation.label.set_text(item.label.text);
                this._itemLocation.selected = item;
                this._itemLocation.selected.setSelected(true);
            }
        }
    },

    _updateSubMenu: function() {
        this.icon.icon_name = !this._weather ? 'view-refresh-symbolic' : this._getIconName(this._weather.item.condition.code, true);
        this._iconLogo.child = this._getIconLogo();
        this._iconWeather.child = this._getIconWeather();
        this._createWeatherForecast();
        this._onShowWeatherForecast();
        this._onWeatherUnits();
    },

    _updateWeather: function() {
        this._loadWeather('SELECT * FROM weather.forecast WHERE woeid="'+this._getLocation(0)+'" and u="f"', function(json) {
            try {
                this._weather = json.query.results.channel;
                if (this._weather.item.condition.code > WEATHER_CODE.length - 1) this._weather.item.condition.code = WEATHER_CODE.length - 1;
                this._updateSubMenu();
                this._setRefreshInterval(this._settings.get_string(KEY_REFRESH_INTERVAL));
            } catch (e) {
                this._setRefreshInterval(5);
            }
        });
    }
});

function init() {
    Convenience.initTranslations();
}

let _extension;

function enable() {
    _extension = PanelTools.addToStatusArea('dschaerfWeather', new WeatherButton(), 1, 'center');
}

function disable() {
    _extension.destroy();
    delete _extension;
}
